--
--  File: C:\PRAG21I\DSGN4_1\lock.vhd
-- created: 04/16/01 13:12:31
--  from: 'C:\PRAG21I\DSGN4_1\lock.asf'
--  by fsm2hdl - version: 2.0.1.53
--
library IEEE;
use IEEE.std_logic_1164.all;

use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

library SYNOPSYS;
use SYNOPSYS.attributes.all;

entity lock is 
  port (clk: in STD_LOGIC;
        rdy: in STD_LOGIC;
        rst: in STD_LOGIC;
        sc: in STD_LOGIC_VECTOR (7 downto 0);
        led: out STD_LOGIC_VECTOR (6 downto 0));
end;

architecture lock_arch of lock is

--diagram signal declarations
signal key1: STD_LOGIC_VECTOR (7 downto 0);
signal key2: STD_LOGIC_VECTOR (7 downto 0);
signal match: STD_LOGIC;
signal newkey1: STD_LOGIC_VECTOR (7 downto 0);
signal newkey2: STD_LOGIC_VECTOR (7 downto 0);

-- ONE HOT ENCODED state machine: Sreg0
type Sreg0_type is (apply, ckkey1, ckkey2, inkey1, inkey2, unlocked, verify1, verify2);
attribute enum_encoding of Sreg0_type: type is
	"00000001 " &		-- apply
	"00000010 " &		-- ckkey1
	"00000100 " &		-- ckkey2
	"00001000 " &		-- inkey1
	"00010000 " &		-- inkey2
	"00100000 " &		-- unlocked
	"01000000 " &		-- verify1
	"10000000";		-- verify2

signal Sreg0: Sreg0_type;

begin
--concurrent signal assignments


Sreg0_machine: process (clk)

begin

if clk'event and clk = '1' then
	if rst='1' then
		Sreg0 <= inkey1;
	else
	case Sreg0 is
		when apply =>
			key1<=newkey1;
			key2<=newkey2;
			if rdy='1' then
				Sreg0 <= ckkey1;
			elsif rdy='0' then
				Sreg0 <= apply;
			end if;
		when ckkey1 =>
			match<='1';
			if rdy='1' then
				Sreg0 <= ckkey2;
				if sc/=key1 then match<='0';
				end if;
			elsif rdy='0' then
				Sreg0 <= ckkey1;
			end if;
		when ckkey2 =>
			if rdy='1' and sc=key2 and match='1' then
				Sreg0 <= unlocked;
			elsif rdy='1' and (sc/=key2 or match='0') then
				Sreg0 <= ckkey1;
			elsif rdy='0' then
				Sreg0 <= ckkey2;
			end if;
		when inkey1 =>
			if rdy='1' then
				Sreg0 <= inkey2;
				newkey1<=sc;
			elsif rdy='0' then
				Sreg0 <= inkey1;
			end if;
		when inkey2 =>
			if rdy='1' then
				Sreg0 <= verify1;
				newkey2<=sc;
			elsif rdy='0' then
				Sreg0 <= inkey2;
			end if;
		when unlocked =>
			if rdy='1' and sc/="01100110" then
				Sreg0 <= ckkey1;
			elsif rdy='1' and sc="01100110" then
				Sreg0 <= inkey1;
			elsif rdy='0' then
				Sreg0 <= unlocked;
			end if;
		when verify1 =>
			if rdy='1' and sc=newkey1 then
				Sreg0 <= verify2;
			elsif rdy='1' and sc/=newkey1 then
				Sreg0 <= ckkey1;
			elsif rdy='0' then
				Sreg0 <= verify1;
			end if;
		when verify2 =>
			if rdy='0' then
				Sreg0 <= verify2;
			elsif rdy='1' and sc/=newkey2 then
				Sreg0 <= ckkey1;
			elsif rdy='1' and sc=newkey2 then
				Sreg0 <= apply;
			end if;
		when others =>   -- trap state
			Sreg0 <= ckkey1;
	end case;
	end if;
end if;
end process;

-- signal assignment statements for combinatorial outputs
led_assignment:
led <= "0001110" when (Sreg0 = apply) else
       "0100101" when (Sreg0 = ckkey1) else
       "0010010" when (Sreg0 = ckkey2) else
       "0100100" when (Sreg0 = inkey2) else
       "0110111" when (Sreg0 = unlocked) else
       "1011101" when (Sreg0 = verify1) else
       "1011011" when (Sreg0 = verify2) else
       "0001100";

end lock_arch;
